@@ -21,16 +21,16 @@
 #include <string.h>
 #include <strings.h>
 
-#ifndef __OpenBSD__
+#if !defined(__OpenBSD__) && !defined(__FreeBSD__)
 # ifndef USE_UNIX98_PTY
 #  include <pty.h>
 # endif /* !USE_UNIX98_PTY */
+# include <termios.h>
 #else
+#if !defined(__FreeBSD__)
 # include <util.h>
 #endif
-
-
-#include <termios.h>
+#endif
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -59,6 +59,7 @@
 #define O_LARGEFILE 0
 #endif
 
+#define SPANDSP_EXPOSE_INTERNAL_STRUCTURES 
 #include <spandsp.h>
 #ifdef STATICLIBS
 #include <iax-client.h>
@@ -141,11 +142,13 @@ static int nodaemon = 0;
 static int commalen = 2;
 static int defskew = 0;
 static int skew;
+static int audiobuflen = 100;
 
 struct modem {
+    struct modem *next;
     int pid;
     char *config;
-    struct modem *next;
+    time_t last_started;
 };
 
 static int numchild;			/* number of children */
@@ -166,22 +169,26 @@ int logmode = S_IRUSR | S_IWUSR | S_IRGRP;
 #include "sys/stropts.h"
 #endif
 
+#ifdef __FreeBSD__
+#include "compat/strndup-freebsd.c"
+#endif
+
 void
 printlog(FILE *fp, char *fmt, ...)
 {
   va_list ap;
   char buf[32];
-  time_t tt;
-  struct tm *ttm;
+  struct timeval tv;
 
-  time(&tt);
-  ttm = localtime(&tt);
-  strftime(buf, sizeof(buf), "[%Y-%m-%d %H:%M:%S] ", ttm);
-  fprintf(fp, "%s", buf);
+  gettimeofday(&tv, NULL);
+  time_t tt = tv.tv_sec;
+  strftime(buf, sizeof(buf), "[%Y-%m-%d %H:%M:%S", localtime(&tt));
+  fprintf(fp, "%s.%.6ld] ", buf, tv.tv_usec);
 
   va_start(ap, fmt);
   vfprintf(fp, fmt, ap);
   va_end(ap);
+  fflush(fp);
 }
 
 void
@@ -421,6 +428,12 @@ setconfigline(char *line)
 	defskew = atoi(line);
 	printlog(LOG_INFO, "Setting skew = %d\n", defskew);
     }
+    if (strncasecmp(line, "audiobuflen", 11) == 0) {
+	line += 11;
+	while (*line == '\t' || *line == ' ') line++;
+	audiobuflen = atoi(line);
+	printlog(LOG_INFO, "Setting audiobuflen = %d\n", audiobuflen);
+    }
 }
 
 void
@@ -530,9 +543,9 @@ t31_call_control_handler(t31_state_t *s, void *user_data, int op, const char *nu
 	    printlog(LOG_INFO, "Answering\n");
 
 	    /* Unset V.24 Circuit 125, "ring indicator". */
-	    int tioflags;
+	    int tioflags = 0;
 	    ioctl(aslave, TIOCMGET, &tioflags);
-	    tioflags |= TIOCM_RI;
+	    tioflags &= ~TIOCM_RI;
 	    ioctl(aslave, TIOCMSET, &tioflags);
 
 	    iax_answer(session[0]);
@@ -565,7 +578,6 @@ t31_call_control_handler(t31_state_t *s, void *user_data, int op, const char *nu
 	case AT_MODEM_CONTROL_HANGUP:
 	    /* Hang up */
 	    printlog(LOG_INFO, "Hanging Up\n");
-	    modemstate = MODEM_ONHOOK;
 	    if (record) {
 		if (dspaudiofd > 0) {
 		    close(dspaudiofd);
@@ -583,12 +595,19 @@ t31_call_control_handler(t31_state_t *s, void *user_data, int op, const char *nu
 	    iaxaudiofd = -1;
 	    gettimeofday(&lasthangup, NULL);
 	    if (phonestate != PHONE_FREED) {
+		if (modemstate == MODEM_RINGING && phonestate == PHONE_CONNECTED ) {
+		    /* Asterisk gets a little confused if we hangup an IAX call that  */
+		    /* we never answered, resulting in congestion, rather than a      */
+		    /* hangup, on the asterisk side.                                  */
+		    iax_answer(session[0]);
+		    sleep(1);	// apparently necessary or the answer can be ignored
+		}
 		iax_hangup(session[0], "Normal disconnect");
 		iax_destroy(session[0]);
 		phonestate = PHONE_FREED;
 		if (gothup) sighandler(SIGHUP);
 	    }
-	    if (modemstate != MODEM_CALLING && modemstate != MODEM_CONNECTED) return -1;
+	    modemstate = MODEM_ONHOOK;
 	    break;
 	case AT_MODEM_CONTROL_CTS:
 	    {
@@ -642,6 +661,20 @@ t31_call_control_handler(t31_state_t *s, void *user_data, int op, const char *nu
     return 0;
 }
 
+void
+simulateHangup(t31_state_t *s)
+{
+    /*
+     * End-of-file was reached.  Simulate hangup.
+     */
+    printlog(LOG_INFO, "Reached end of recording to replay.  Simulating hangup.\n");
+    phonestate = PHONE_FREED;
+    modemstate = MODEM_ONHOOK;
+    t31_call_event(s, AT_CALL_EVENT_HANGUP);
+    iax_hangup(session[0], "Normal disconnect");
+    if (gothup) sighandler(SIGHUP);
+}
+
 void
 iaxmodem(const char *config, int nondaemon)
 {
@@ -649,8 +682,8 @@ iaxmodem(const char *config, int nondaemon)
      * IAXmodem is started as root, but drops privileges once the
      * required superuser tasks are performed.
      */
-    uid_t uucp_uid;
-    gid_t uucp_gid;
+    uid_t modem_uid;
+    gid_t modem_gid;
     struct passwd *pwent;
     int fd;
     char logfile[256];
@@ -664,8 +697,8 @@ iaxmodem(const char *config, int nondaemon)
 	_exit(-1);
     }
 
-    uucp_uid = pwent->pw_uid;
-    uucp_gid = pwent->pw_gid;
+    modem_uid = pwent->pw_uid;
+    modem_gid = pwent->pw_gid;
 
     strcpy(devlink, "/dev/iaxmodem");
     strcpy(server, "127.0.0.1");
@@ -729,7 +762,7 @@ iaxmodem(const char *config, int nondaemon)
     int16_t iaxbuf[VOIP_PACKET_SIZE];
     static t31_state_t t31_state;
     int t31buflen;
-    int tioflags;
+    int tioflags = 0;
     struct group *grent;
     char *devgroup;
     char *pmode;
@@ -739,6 +772,10 @@ iaxmodem(const char *config, int nondaemon)
     int i;
     unsigned int last_ts = 0;
 
+    int16_t audiobuf[VOIP_PACKET_SIZE*audiobuflen];
+    int16_t* audiobufpos = audiobuf;
+    memset(audiobuf, (int16_t) 0, sizeof(int16_t)*VOIP_PACKET_SIZE*audiobuflen);
+
     snprintf(dspnowaudiofile, sizeof(dspnowaudiofile), "/tmp/%s-dsp.raw.recording", regpeer);
     snprintf(iaxnowaudiofile, sizeof(iaxnowaudiofile), "/tmp/%s-iax.raw.recording", regpeer);
     snprintf(dspaudiofile, sizeof(dspaudiofile), "/tmp/%s-dsp.raw", regpeer);
@@ -874,8 +911,8 @@ iaxmodem(const char *config, int nondaemon)
     }
 
     /* Root privileges not needed anymore, drop privs. */
-    setegid(uucp_gid);
-    seteuid(uucp_uid);
+    setegid(modem_gid);
+    seteuid(modem_uid);
     
     if ((port = iax_init(port) < 0)) {
 	printlog(LOG_ERROR, "Fatal error: failed to initialize iax with port %d\n", port);
@@ -908,10 +945,13 @@ iaxmodem(const char *config, int nondaemon)
      * Again, debugging should be minimal for normal use as the writes slow things too much.
      */
     if (dspdebug) {
-	t31_state.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
-	t31_state.audio.modems.v17_rx.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
-	t31_state.audio.modems.v29_rx.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
-	t31_state.audio.modems.v27ter_rx.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
+	int loglevel = SPAN_LOG_SEVERITY_MASK | SPAN_LOG_SHOW_DATE | SPAN_LOG_SHOW_SEVERITY | 
+	    SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_SHOW_VARIANT | SPAN_LOG_SHOW_TAG;
+	t31_state.logging.level = loglevel;
+	t31_state.at_state.logging.level = loglevel;
+	t31_state.audio.modems.fast_modems.v17_rx.logging.level = loglevel;
+	t31_state.audio.modems.fast_modems.v29_rx.logging.level = loglevel;
+	t31_state.audio.modems.fast_modems.v27ter_rx.logging.level = loglevel;
     }
 
     int selectfd, selectretval, selectblock, avail, audiobalance = 0;
@@ -957,15 +997,37 @@ iaxmodem(const char *config, int nondaemon)
 	    tv.tv_sec = 5; tv.tv_usec = 0;	/* ring every 5 seconds */
 	    timeradd(&lastring, &tv, &tv);
 	    timersub(&tv, &now, &tv);
+	    if (tv.tv_sec > 100 || tv.tv_sec < -100) {
+		/* This is a silly result.  The clock must have skipped. */
+		printlog(LOG_INFO, "Clock skip detected in ringing (%d seconds).  Compensating.\n", tv.tv_sec);
+		lastring = now;
+		tv.tv_sec = 5;
+		tv.tv_usec = 0;
+	    }
 	} else if (modemstate == MODEM_CALLING) {
-	    tv.tv_sec = 45; tv.tv_usec = 0;	/* give up after 45 seconds */
+	    tv.tv_sec = (t31_state.at_state.p.s_regs[7] + 5); 	/* give up after S7+5 seconds */
+	    tv.tv_usec = 0;
 	    timeradd(&lastdtedata, &tv, &tv);
 	    timersub(&tv, &now, &tv);
+	    if (tv.tv_sec > 500 || tv.tv_sec < -500) {
+		/* This is a silly result.  The clock must have skipped. */
+		lastdtedata = now;
+		printlog(LOG_INFO, "Clock skip detected in DTE communication (%d seconds).  Compensating.\n", tv.tv_sec);
+		tv.tv_sec = (t31_state.at_state.p.s_regs[7] + 5);
+		tv.tv_usec = 0;
+	    }
 	} else if (refreshreq) {
 	    /* Handle maintaining registration every refresh - 5 seconds. */
 	    tv.tv_sec = refresh - 5; tv.tv_usec = 0;
 	    timeradd(&lastreg, &tv, &tv);
 	    timersub(&tv, &now, &tv);
+	    if (tv.tv_sec > 10000 || tv.tv_sec < -10000) {
+		/* This is a silly result.  The clock must have skipped. */
+		printlog(LOG_INFO, "Clock skip detected in IAX2 registration (%d seconds).  Compensating.\n", tv.tv_sec);
+		lastreg = now;
+		tv.tv_sec = refresh - 5;
+		tv.tv_usec = 0;
+	    }
 	} else if (iax_time_to_next_event() >= 0) {
 	    /* The actual calculations will be done later. */
 	    tv.tv_sec = 60; tv.tv_usec = 0;
@@ -1048,7 +1110,11 @@ iaxmodem(const char *config, int nondaemon)
 		} while (t31buflen < VOIP_PACKET_SIZE && gotlen > 0 && modemstate == MODEM_CONNECTED);
 	    }
 	    if (record) write(dspaudiofd, (uint8_t *) dspbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
-	    if (replay) read(dspaudiofd, (uint8_t *) dspbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
+	    if (replay) {
+		if (read(dspaudiofd, (uint8_t *) dspbuf, VOIP_PACKET_SIZE*sizeof(int16_t)) == 0) {
+		    simulateHangup(&t31_state);
+		}
+	    }
 	    if (modemstate == MODEM_CONNECTED) {	/* t31_tx can change modemstate */
 		audiobalance++;
 		if (audiobalance > 5) {
@@ -1095,7 +1161,7 @@ iaxmodem(const char *config, int nondaemon)
 	    (modemstate == MODEM_CONNECTED && !t31_state.tx.holding && avail)) {
 	    ssize_t len;
 	    do {
-		len = read(amaster, modembuf, avail);
+		len = read(amaster, modembuf, 1);
 		if (len > 0) {
 		    int taken = t31_at_rx(&t31_state, modembuf, len);
 		    if (taken != len) {
@@ -1105,7 +1171,6 @@ iaxmodem(const char *config, int nondaemon)
 			   timing problems. */
 			printlog(LOG_ERROR, "Unexpected modem buffering. Sent %zd bytes, modem buffered %d.\n", len, taken);
 		    }
-		    len -= taken;	/* ??? */
 		    avail -= taken;
 		    lastdtedata = now;
 		}
@@ -1120,8 +1185,12 @@ iaxmodem(const char *config, int nondaemon)
 	}
 	/*
 	 * Did the DCE timeout in sending CONNECT response?
+	 * spandsp has an S-register (7) that is employed there.
+	 * We merely offer a fail-over 5 seconds later if that 
+	 * doesn't work
+	 *
 	 */
-	if (modemstate == MODEM_CALLING && timediff(now, lastdtedata) >= 45000000) {
+	if (modemstate == MODEM_CALLING && timediff(now, lastdtedata) >= (t31_state.at_state.p.s_regs[7] + 5) * 1000000) {
 	    t31_call_event(&t31_state, AT_CALL_EVENT_NO_ANSWER);
 	    /*
 	     * One would think that this would be appropriate here:
@@ -1143,7 +1212,8 @@ iaxmodem(const char *config, int nondaemon)
 	 */
 	if ((modemstate != MODEM_CONNECTED && selectretval && FD_ISSET(iaxnetfd, &select_rfds)) ||
 	    modemstate == MODEM_CONNECTED || !iax_time_to_next_event()) {
-	    while ((iaxevent = iax_get_event(0))) {
+	    int times = 0;	// avoid a bug in libiax2 where iax_get_event() always retrieves voice data even after a call ends
+	    while (times++ < 100 && (iaxevent = iax_get_event(0))) {
 		switch (iaxevent->etype) {
 #ifdef IAX_EVENT_NULL
 		    case IAX_EVENT_NULL:
@@ -1193,6 +1263,7 @@ iaxmodem(const char *config, int nondaemon)
 			}
 			modemstate = MODEM_CONNECTED;
 			audiobalance = 0;
+			audiobufpos = audiobuf;
 			skew = defskew;
 			if (record) {
 			    if (dspaudiofd > 0) {
@@ -1240,6 +1311,7 @@ iaxmodem(const char *config, int nondaemon)
 				}
 			    }
 			}
+			audiobufpos = audiobuf;
 			audiobalance = 0;
 			skew = defskew;
 			last_ts = 0;
@@ -1277,7 +1349,7 @@ iaxmodem(const char *config, int nondaemon)
 			if (gothup) sighandler(SIGHUP);
 			break;
 		    case IAX_EVENT_HANGUP:
-			printlog(LOG_INFO, "Remote hangup.\n");
+			printlog(LOG_INFO, "Remote hangup. Cause: \"%s\" Code: %u\n", iaxevent->ies.cause ? iaxevent->ies.cause : "", iaxevent->ies.causecode);
 			if (modemstate != MODEM_ONHOOK) {
 			    if (modemstate == MODEM_CALLING)
 				t31_call_event(&t31_state, AT_CALL_EVENT_BUSY);
@@ -1305,8 +1377,12 @@ iaxmodem(const char *config, int nondaemon)
 			break;
 		    case IAX_EVENT_CNG:
 			/* pseudo-silence */
-			memset(dspbuf, (int16_t) 0, VOIP_PACKET_SIZE);
-			if (replay) read(iaxaudiofd, (uint8_t *) dspbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
+			memset(dspbuf, (int16_t) 0, sizeof(int16_t)*VOIP_PACKET_SIZE);
+			if (replay) {
+			    if (read(iaxaudiofd, (uint8_t *) dspbuf, VOIP_PACKET_SIZE*sizeof(int16_t)) == 0) {
+				simulateHangup(&t31_state);
+			    }
+			}
 			if (!dialextra && modemstate == MODEM_CONNECTED && t31_rx(&t31_state, (int16_t *) dspbuf, VOIP_PACKET_SIZE)) {
 			    printlog(LOG_ERROR, "Error sending silence to DSP.\n");
 			}
@@ -1317,6 +1393,11 @@ iaxmodem(const char *config, int nondaemon)
 			 * Watch for IAX2 jitter... the 32-bit timestamp shouldn't ever "wrap" 
 			 * unless we expect a single call to last more than a month.
 			 */
+			if (!nojitterbuffer && last_ts && abs((int)(iaxevent->ts - last_ts)) > 300  ) {
+			    printlog(LOG_ERROR, "IAX2 jitter gap too big, syncing - last_ts: %u, ts: %u\n", last_ts, iaxevent->ts);
+			    last_ts = iaxevent->ts;
+			    break;
+			}
 			if (!nojitterbuffer && last_ts && iaxevent->ts <= last_ts) {
 			    /*
 			     * We've already sent audio corresponding to this time.
@@ -1324,7 +1405,7 @@ iaxmodem(const char *config, int nondaemon)
 			    break;
 			}
 			if (!nojitterbuffer && last_ts && iaxevent->ts != last_ts + 20) {
-			    printlog(LOG_ERROR, "IAX2 jitter - last_ts: %d, ts: %d\n", last_ts, iaxevent->ts);
+			    printlog(LOG_ERROR, "IAX2 jitter - last_ts: %u, ts: %u\n", last_ts, iaxevent->ts);
 			    int16_t fillbuf[VOIP_PACKET_SIZE];
 			    memcpy(fillbuf, iaxbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
 			    while (last_ts + 20 < iaxevent->ts) {
@@ -1338,9 +1419,13 @@ iaxmodem(const char *config, int nondaemon)
 				 * not replaced with non-silence... in order to keep as
 				 * syncrhonous with the remote as possible.
 				 */
-				if (replay) read(iaxaudiofd, fillbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
+				if (replay) {
+				    if (read(iaxaudiofd, fillbuf, VOIP_PACKET_SIZE*sizeof(int16_t)) == 0) {
+					simulateHangup(&t31_state);
+				    }
+				}
 				if (!dialextra && modemstate == MODEM_CONNECTED && t31_rx(&t31_state, fillbuf, VOIP_PACKET_SIZE)) {
-				    printlog(LOG_ERROR, "Error sending %d units of IAX make-up audio to DSP.\n", VOIP_PACKET_SIZE);
+				    printlog(LOG_ERROR, "Error sending %d samples of IAX make-up audio to DSP.\n", VOIP_PACKET_SIZE);
 				}
 				if (record) write(iaxaudiofd, fillbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
 				last_ts += 20;
@@ -1367,14 +1452,14 @@ iaxmodem(const char *config, int nondaemon)
 			}
 			/* Here's an audio frame from the IAX server, send it to the DSP. */
 			{
-			    int units = iaxevent->datalen;
+			    int samples = iaxevent->datalen;
 			    int16_t convertedbuf[iaxevent->datalen];
 			    int16_t* audiodata;
 			    int i = 0;
 			    switch (codec) {
 				case AST_FORMAT_SLINEAR:
-				    units = iaxevent->datalen/sizeof(int16_t);
-				    orderbytes((int16_t *) iaxevent->data, units);
+				    samples = iaxevent->datalen/sizeof(int16_t);
+				    orderbytes((int16_t *) iaxevent->data, samples);
 				    audiodata = (int16_t*) iaxevent->data;
 				    break;
 				case AST_FORMAT_ALAW:
@@ -1386,15 +1471,62 @@ iaxmodem(const char *config, int nondaemon)
 				    audiodata = convertedbuf;
 				    break;
 				default:
+				    samples = 0;
+				    memset(convertedbuf, 0, iaxevent->datalen);
+				    audiodata = convertedbuf;
 				    printlog(LOG_ERROR, "Unknown codec!\n");
 				    break;
 			    }
-			    memcpy(iaxbuf, audiodata, units >= VOIP_PACKET_SIZE ? VOIP_PACKET_SIZE*sizeof(int16_t) : units*sizeof(int16_t));
-			    if (replay) read(iaxaudiofd, audiodata, units*sizeof(int16_t));
-			    if (!dialextra && modemstate == MODEM_CONNECTED && t31_rx(&t31_state, audiodata, units)) {
-				printlog(LOG_ERROR, "Error sending %d units of IAX audio to DSP.\n", units);
+
+			    /* Save this for our use in with jitter-fill. */
+			    if (!nojitterbuffer)
+				memcpy(iaxbuf, audiodata, samples >= VOIP_PACKET_SIZE ? VOIP_PACKET_SIZE*sizeof(int16_t) : samples*sizeof(int16_t));
+
+			    /*
+			     * During a fax session (the entire time the modem is off-hook) the modem should
+			     * only be in command-mode for extemely brief periods of time (between commands).
+			     * Whatever audio we send to the DSP while the modem is in command-mode is lost 
+			     * and wasted.  Sometimes the DTE is slow between one command's result and the 
+			     * sending of the subsequent command.  Buffer this audio to some extent and "replay" 
+			     * it later to the DSP once we leave command-mode.
+			     */
+			    if (t31_state.at_state.at_rx_mode == AT_MODE_OFFHOOK_COMMAND ||
+				t31_state.at_state.at_rx_mode == AT_MODE_ONHOOK_COMMAND) {
+				/* Save this audio for later. */
+				if ((audiobufpos + samples) > (audiobuf + VOIP_PACKET_SIZE*audiobuflen)) {
+				    int16_t bufsamples = audiobufpos - (audiobuf + samples);
+				    memmove(audiobuf, audiobuf+samples, bufsamples*sizeof(int16_t));
+				    audiobufpos = audiobuf + bufsamples;
+				}
+				memcpy(audiobufpos, audiodata, samples*sizeof(int16_t));
+				audiobufpos += samples;
+				/* Use silence instead. */
+				memset(audiodata, (int16_t) 0, samples*sizeof(int16_t));
+			    } else {
+				if (audiobufpos != audiobuf && !dialextra && modemstate == MODEM_CONNECTED) {
+				    /* Deliver audio that was buffered while the modem was in command-mode. */
+				    printlog(LOG_INFO, "DTE was slow.  Playing %ld samples of buffered audio.\n", audiobufpos-audiobuf);
+				    if (replay) {
+					if (read(iaxaudiofd, audiobuf, audiobufpos-audiobuf) == 0) {
+					    simulateHangup(&t31_state);
+					}
+				    }
+				    if (t31_rx(&t31_state, audiobuf, (audiobufpos-audiobuf)/sizeof(int16_t))) {
+					printlog(LOG_ERROR, "Error sending %ld samples of buffered IAX audio to DSP.\n", audiobufpos-audiobuf);
+				    }
+				    if (record) write(iaxaudiofd, audiobuf, audiobufpos-audiobuf);
+				    audiobufpos = audiobuf;
+				}
+				if (replay) {
+				    if (read(iaxaudiofd, audiodata, samples*sizeof(int16_t)) == 0) {
+					simulateHangup(&t31_state);
+				    }
+				}
+				if (record) write(iaxaudiofd, audiodata, samples*sizeof(int16_t));
+			    }
+			    if (!dialextra && modemstate == MODEM_CONNECTED && t31_rx(&t31_state, audiodata, samples)) {
+				printlog(LOG_ERROR, "Error sending %d samples of IAX audio to DSP.\n", samples);
 			    }
-			    if (record) write(iaxaudiofd, audiodata, units*sizeof(int16_t));
 			}
 			break;
 		    case IAX_EVENT_TRANSFER:
@@ -1448,89 +1580,80 @@ iaxmodem(const char *config, int nondaemon)
     }
 } 
 
-void
-add_modem(int pid, char *config)
-{
+struct modem *modem_for_config(const char *config) {
     struct modem *m;
+
+    for (m = modems; (m != NULL) && (strcmp(m->config, config) != 0); m = m->next) /* just walking the list... */;
+    return m;
+}
+
+struct modem *modem_for_pid(pid_t pid) {
+    struct modem *m;
+
+    for (m = modems; (m != NULL) && (m->pid != pid); m = m->next) /* just walking the list... */;
+    return m;
+}
+
+void
+add_modem(char *config)
+{
     struct modem *nm;
 
     nm = malloc(sizeof(struct modem));
 
-    for (m = modems; (m != NULL) && (m->next != NULL); m = m->next);
-
-    nm->pid = pid;
-    nm->config = config;
-    nm->next = NULL;
-
-    if (m != NULL)
-      m->next = nm;
-    else
-      modems = nm;
+    nm->pid = 0;
+    nm->last_started = 0;
+    nm->config = strdup(config);
+    nm->next = modems;
+    modems = nm;
 
     numchild++;
 }
 
 void
-restart_modem(pid_t pid)
+remove_modem(struct modem **m)
 {
-    struct modem *m;
-
-    for (m = modems; m != NULL; m = m->next) {
-	if (m->pid == pid) {
-	    pid_t newpid = fork();
-	    if (newpid == 0) {
-		/* child */
-		iaxmodem(m->config, 0);
-		_exit(255);		/* shouldn't ever get here */
-	    } else if (newpid > 0) {
-		/* parent */
-		m->pid = newpid;
-	    } else {
-		/* failed */
-		printlog(LOG_ERROR, "Error: fork failed: %s\n", strerror(errno));
-	    }
-	    break;
-	}
-    }
-}
-
-void
-remove_modem(int pid)
-{
-    struct modem *m;
-    struct modem *p = NULL;
-
-    for (m = modems; (m != NULL) && (m->next != NULL); p = m,  m = m->next)
-      {
-	if (m->pid == pid) {
-	  if (p != NULL)
-	    p->next = m->next;
-
-	  if (m == modems)
-	    modems = NULL;
-
-	  free(m->config);
-	  free(m);
-	  numchild--;
-
-	  break;
-	}
-      }
+    struct modem *n = (*m)->next;
+    free((*m)->config);
+    free(*m);
+    *m = n;
+    numchild--;
 }
 
 void
 remove_all_modems()
 {
-    struct modem *m;
-    struct modem *p = NULL;
+    while (numchild) 
+	remove_modem(&modems);
+}
 
-    for (m = modems; m != NULL; p = m,  m = m->next)
-      {
-	free(p->config);
-	free(p);
-      }
+void start_modem(struct modem *m) {
+    pid_t newpid = fork();
+    if (newpid == 0) {
+        /* child */
+        const char *config=strdup(m->config);
+	remove_all_modems();    /* Children don't need info about siblings */
+        iaxmodem(config, 0);    /* Fax away */
+        _exit(255);		/* shouldn't ever get here */
+    } else if (newpid > 0) {
+        /* parent */
+        printlog(LOG_ERROR, "Process %d started for config %s.\n", newpid, m->config);
+        m->pid = newpid;
+        m->last_started = time(NULL);
+    } else {
+        /* failed */
+        printlog(LOG_ERROR, "Error: no modem could be started for %s (fork failed: %s)\n", m->config, strerror(errno));
+    }
+}
 
-    modems = NULL;
+void
+respawn_modem(struct modem *m)
+{
+    if (time(NULL)-m->last_started < 5) {
+        printlog(LOG_INFO, "Process for config file %s respawning too fast, not restarting.\n", m->config);
+    } else {
+        start_modem(m);
+    }
 }
 
 void
@@ -1539,23 +1662,22 @@ wait_for_modems()
     pid_t pid;
     int status;
 
-    while (numchild > 0) {
-	/* Wait for any child to exit */
-	pid = waitpid(-1, &status, 0);
-
-	if (pid > 0) {
-	    if ((status & 0xFF) == 0x00) status >>= 8;
-	    if (status == SIGHUP) {
-		printlog(LOG_ERROR, "iaxmodem process %d ended w/SIGHUP, attempting to restart\n", pid);
-		restart_modem(pid);
-	    } else {
-		printlog(LOG_ERROR, "iaxmodem process %d ended, status 0x%X\n", pid, status);
-		remove_modem(pid);
-	    }
+    /* Wait for any child to exit */
+    while ((pid = waitpid(-1, &status, 0)) > 0) {
+	struct modem *m = modem_for_pid(pid);
+	if ((status & 0xff) == 0) {
+	    printlog(LOG_ERROR, "iaxmodem process %d terminated on %s.\n", pid, strsignal(status>>8));
 	} else {
-	    break;
+	    printlog(LOG_ERROR, "iaxmodem process %d ended, status 0x%X.\n", pid, status);
+	}
+	if (m == NULL) {
+	    printlog(LOG_ERROR, "forgot all about iaxmomdem process %d.\n", pid);
+	} else {
+	    printlog(LOG_ERROR, "respawning process for config: %s.\n", m->config);
+	    respawn_modem(modem_for_pid(pid));
 	}
     }
+    printlog(LOG_ERROR, "waitpid returned: %d\n", pid);
 }
 
 void
@@ -1564,141 +1686,94 @@ terminate_modems()
     struct modem *m;
 
     for (m = modems; m != NULL; m = m->next)
-      {
 	kill(m->pid, SIGTERM);
-      }
 }
 
 void
-reload_modems()
-{
-    struct modem *m;
-
-    for (m = modems; m != NULL; m = m->next)
-      {
-	kill(m->pid, SIGHUP);
-      }
-}
-
-int
 spawn_modems(void)
 {
     DIR *cfdir;
     struct dirent *cf;
     struct stat st;
     int len;
-    int pid = -1;
-    char *config = NULL;
     char filename[256];
+    struct modem *m, **mp;
 
+    /* purge existing list */
+    mp = &modems;
+    while (*mp != NULL) {
+	m = *mp;
+	snprintf(filename, sizeof(filename), "/etc/iaxmodem/%s", m->config);
+        if (stat(filename, &st) < 0)
+            printlog(LOG_INFO, "Config file %s disappeared (%s), forgetting about it.\n", filename, strerror(errno));
+        else if (!S_ISREG(st.st_mode))
+            printlog(LOG_INFO, "Config file %s was replaced by a non-regular file, forgetting about it.\n", filename);
+        else {
+	    mp = &(m->next);
+	    continue;
+	}
+        if (m->pid != 0) kill(m->pid, SIGTERM);
+        remove_modem(mp);
+    }
+    
     /* List configuration files */
     cfdir = opendir("/etc/iaxmodem");
 
     if (cfdir == NULL) {
       printlog(LOG_ERROR, "Error: could not open configuration directory: %s\n", strerror(errno));
-      return -1;
+      return;
     }
 
-    while ((cf = readdir(cfdir)) != NULL)
-      {
+    while ((cf = readdir(cfdir)) != NULL) {
 	/* Skip dotfiles and backup files */
 	len = strlen(cf->d_name);
 	if ((cf->d_name[0] == '.') || (cf->d_name[0] == '#') || (cf->d_name[len - 1] == '~'))
-	  continue;
+            continue;
 
 	/* Skip anything that isn't a file */
 	snprintf(filename, sizeof(filename), "/etc/iaxmodem/%s", cf->d_name);
-	if (stat(filename, &st) < 0)
-	  {
-	    printlog(LOG_INFO, "Could not stat configuration file %s: %s\n", cf->d_name, strerror(errno));
+	if (stat(filename, &st) < 0) {
+            printlog(LOG_INFO, "Could not stat configuration file %s: %s\n", cf->d_name, strerror(errno));
 	    continue;
-	  }
-
-	if (!S_ISREG(st.st_mode))
-	  {
-	    printlog(LOG_INFO, "bouh !\n");
+        }
+	if (!S_ISREG(st.st_mode)) {
+	    printlog(LOG_INFO, "%s: not a regular file, skipping.\n");
 	    continue;
-	  }
-
-	config = strdup(cf->d_name);
-
-	/* Spawn modem processes */
-	pid = fork();
-	if (pid == 0) {
-	  /* Child, get out of this loop */
-	  break;
-	} else if (pid > 0) {
-	  /* Controlling process */
-	  add_modem(pid, config);
-	} else {
-	  /* Failed */
-	  printlog(LOG_ERROR, "Error: fork failed: %s\n", strerror(errno));
-
-	  if (numchild > 0) {
-	    printlog(LOG_ERROR, "%d children spawned, continuing anyway (stopped at %s)\n", numchild, config);
-	  } else {
-	    return -1;
-	  }
-
-	  free(config);
-	  break;
-	}
-      }
-
+        }
+        if (modem_for_config(cf->d_name) == NULL)
+            add_modem(cf->d_name);
+    }
     closedir(cfdir);
-
-    if (pid == 0) {
-      /* Start the modem */
-      iaxmodem(config, 0);
-
-      return 1;
+    
+    for (m = modems; m != NULL; m = m->next) {
+        if (m->pid != 0 && kill(m->pid, 0)==0) /* child still running */
+            kill(m->pid, SIGHUP);
+        else
+            start_modem(m);
     }
-
-    return 0;
 }
 
 void
-ctrl_sighandler(int sig)
+ctrl_term_sighandler(int sig)
 {
-    signal(SIGHUP, NULL);
-    signal(SIGTERM, NULL);
-
-    printlog(LOG_ERROR, "Terminating...\n");
-
-    /* Terminate children */
-    terminate_modems();
+    /* nothing to do... the signal will interrupt waitpid, without gothup == 1
+       and that will cause the program to terminate. */
 }
 
 void
 ctrl_hup_sighandler(int sig)
 {
-    int fd;
-
     gothup = 1;
-
-    printlog(LOG_ERROR, "Configuration changed, restarting...\n");
-
-    /* Reopen the log file */
-    checklog("/var/log/iaxmodem/iaxmodem");
-    fd = open("/var/log/iaxmodem/iaxmodem", O_WRONLY | O_APPEND | O_CREAT | O_LARGEFILE, logmode);
-    if (fd < 0) {
-      printlog(LOG_ERROR, "Error: could not open /var/log/iaxmodem/iaxmodem: %s\n", strerror(errno));
-    } else {
-      dup2(fd, STDERR_FILENO);
-    }
-    close(fd);  
-
-    reload_modems();
 }
 
 int
 main(int argc, char** argv)
 {
-    int ret;
     char config[256];
     int isdaemon = 1;
     int fd;
     FILE *pidfile;
+    struct sigaction sa;
 
     if (geteuid() != 0) {
       printlog(LOG_ERROR, "Error: run iaxmodem as root\n");
@@ -1747,8 +1822,12 @@ main(int argc, char** argv)
 
     gothup = 1;
 
-    signal(SIGTERM, ctrl_sighandler);
-    signal(SIGHUP, ctrl_hup_sighandler);
+    sa.sa_handler = ctrl_term_sighandler;
+    sigemptyset(&sa.sa_mask);
+    sa.sa_flags = 0;
+    sigaction(SIGTERM, &sa, NULL);
+    sa.sa_handler = ctrl_hup_sighandler;
+    sigaction(SIGHUP, &sa, NULL);
 
     /* No config file specified, be a controlling process for modems */
     numchild = 0;
@@ -1766,31 +1845,28 @@ main(int argc, char** argv)
     }
     close(fd);
 
-    /* Redirect stderr to the log file */
-    checklog("/var/log/iaxmodem/iaxmodem");
-    fd = open("/var/log/iaxmodem/iaxmodem", O_WRONLY | O_APPEND | O_CREAT | O_LARGEFILE, logmode);
-    if (fd < 0) {
-      printlog(LOG_ERROR, "Error: could not open /var/log/iaxmodem/iaxmodem: %s\n", strerror(errno));
-    } else {
-      dup2(fd, STDERR_FILENO);
-    }
-    close(fd);
-
-    while (gothup)
-      {
-	ret = spawn_modems();
+    while (gothup) {
 	gothup = 0;
 
-	if (ret == 0) {
-	  wait_for_modems();
-	} else if (ret < 0) {
-	  exit(-1);
+	/* Reopen the log file */
+	checklog("/var/log/iaxmodem/iaxmodem");
+	fd = open("/var/log/iaxmodem/iaxmodem", O_WRONLY | O_APPEND | O_CREAT | O_LARGEFILE, logmode);
+	if (fd < 0) {
+	    printlog(LOG_ERROR, "Error: could not open /var/log/iaxmodem/iaxmodem: %s\n", strerror(errno));
 	} else {
-	  break;
+	    dup2(fd, STDERR_FILENO);
 	}
-      }
+	close(fd);
 
-    /* Control process only */
+	/* kill/spawn/restart children */
+	spawn_modems();
+	if (numchild <= 0)
+	    break;
+	wait_for_modems();
+    }
+
+    printlog(LOG_ERROR, "Terminating...\n");
+    terminate_modems();
     unlink("/var/run/iaxmodem.pid");
 
     exit(0);
